static char rcsid[] = "@(#)$Id: shared_connect.c,v 2.4 2015/11/08 09:31:46 hurtta Exp $";

/******************************************************************************
 *  The Elm (ME+) Mail System  -  $Revision: 2.4 $   $State: Exp $
 *
 *  Author: Kari Hurtta <hurtta+elm@siilo.FMI.FI> (was hurtta+elm@posti.FMI.FI)
 *      or  Kari Hurtta <elm@elmme-mailer.org>
 *****************************************************************************/

/*
 * This file is compiled only if dlopen() is available, so
 * that file does not need to be guarded with #ifdef 
 */

#include "elm_defs.h"

DEBUG_VAR(Debug,__FILE__,"dl");

#include "shared_imp.h"
#include "rc_imp.h"
#include "save_opts.h"
#include "cs_imp.h"
#include "connection_imp.h"
#include "ss_imp.h"

#if ANSI_C
#define S_(x) static x;
#else
#define S_(x)
#endif

#define SHARED_CONNECT_magic	0xF901

S_(provides_initial_TLS_ra_f no_initial_TLS_ra)
static enum tls_ra_status no_initial_TLS_ra(ra,proto_name,
					    service_config,
					    sc_size)
    struct remote_account * ra;
    const char            * proto_name; /* for messages */
    struct SE_option      * service_config;
     size_t                  sc_size; 
{

    return tls_ra_unsupported;
}



struct sl_connect_data {
    uint16            magic;      

    provides_initial_TLS_ra_f * initial_TLS_ra;

};



S_(sl_reg_functions sl_reg_connect)
static int sl_reg_connect P_((struct ImpInfo *i,
			      int reg_idx));
static int sl_reg_connect(i,reg_idx)
     struct ImpInfo *i;
     int reg_idx;
{
    /* union hack to avoid warnings about casting
     * between pointer-to-object and pointer-to-function
     */

    int res = 0;

    
    union F6 {
	void *                       ptr;
	provides_shared_SEOT_f     * f6;
    } f6;

    union F15 {
	void                      * ptr;
	provides_initial_TLS_ra_f * f15; 
    } f15;

    struct reg_list *r = & (i->regs[reg_idx]);

    if (&use_shared_connect != r->var)
	panic("SHARED PANIC",__FILE__,__LINE__,
	      "sl_reg_connect",
	      "Wrong variable",0);
    
    if (r->r.connect->magic != SHARED_CONNECT_magic)
	panic("SHARED PANIC",__FILE__,__LINE__,
	      "sl_reg_connect",
	      "Bad magic bumber",0);

    f6.ptr  = dlsym(i->handle,  "provides_shared_SEOT");
    f15.ptr = dlsym(i->handle,  "provides_initial_TLS_ra");

    if (!f6.f6) {
	DPRINT(Debug,7,(&Debug, " ... NO provides_shared_SEOT\n"));
    }

    if (!f15.f15) {
	DPRINT(Debug,7,(&Debug, " ... NO provides_initial_TLS_ra\n"));
    }



#ifdef REMOTE_MBX


    if (f6.f6) {
	    int count,x;
	    size_t   s_res6;
	    struct SE_option_type ** res6 = f6.f6(&count, &s_res6);

	    if (s_res6 != sizeof (**res6)) {
		DPRINT(Debug,1,(&Debug,"... struct SE_option_type mismatch: %d should be %d\n",
				s_res6,sizeof (**res6)));
	    } else {
		shared_SE_option_types = 
		    safe_realloc(shared_SE_option_types,
				 sizeof (shared_SE_option_types[0]) *
				 (shared_SE_option_type_count + count));

		DPRINT(Debug,7,(&Debug," ... provides_shared_SEOT:  (shared_SE_option_types) count %d\n",
				count));
		res = 1;
		
		for (x = 0; x < count; x++) {

		    if (res6[x]->magic != SE_option_t_magic) 
			panic("SHARED PANIC",__FILE__,__LINE__,
			      "sl_reg_connect",
			      "bad magic number on provides_shared_SEOT",0);


		    shared_SE_option_types[shared_SE_option_type_count
					  +x].T = res6[x];
		    shared_SE_option_types[shared_SE_option_type_count
					  +x].imp_idx = i;
		}
		shared_SE_option_type_count += count;
	    }
    }

    if (f15.f15) {
	r->r.connect->initial_TLS_ra = f15.f15;
	DPRINT(Debug,7,(&Debug, " ... provides_initial_TLS_ra\n"));
	res = 1;
    }

#endif
	

    i->regs[reg_idx].valid   = res;
    
    DPRINT(Debug,7,(&Debug, 
		    "sl_reg_connect:  [%p]->regs[%d].valid = %d\n",
		    i,reg_idx,res));

   return res;
}

S_(sl_zero_reg_list sl_zero_connect)
static void sl_zero_connect P_((struct dt_shared_info *var,
				struct reg_list *r));
static void sl_zero_connect(var,r)
     struct dt_shared_info *var;
     struct reg_list *r;
{
    r->var     = var;
    r->valid   = 0;
    r->r.connect  = safe_malloc(sizeof (*r->r.connect));   /* never freed ... */
    r->r.connect->magic = SHARED_CONNECT_magic;
    r->r.connect->initial_TLS_ra = no_initial_TLS_ra;
}

S_(sl_unreg_functions sl_unreg_connect)
static void sl_unreg_connect P_((struct ImpInfo *i, int reg_idx));
static void sl_unreg_connect(i,reg_idx)
     struct ImpInfo *i; 
     int reg_idx;
{
    struct reg_list *r = & (i->regs[reg_idx] );
    r->valid   = 0;

}

static struct shared_loader connect_loader = {
    SHARED_LOADER_magic,
    sl_reg_connect,
    sl_zero_connect,
    sl_unreg_connect
};

SHAREDLIB use_shared_connect = {
    &connect_loader            /* loader */,
    0,                       
    NULL, 0,

    NULL
};

#ifdef REMOTE_MBX

enum itls_status initial_TLS_ra(ra,proto_name)
     struct remote_account * ra;
     const char            * proto_name; /* for messages */
{
    enum tls_ra_status ret = tls_ra_unsupported;
    enum tls_ra_status sticky_ret = tls_ra_unsupported;
    int i;

    if (REMOTE_ACCOUNT_magic != ra->magic) {
	panic("CONNECTION PANIC",__FILE__,__LINE__,
	      "initial_TLS_ra",
	      "Bad magic (remote_account)",0);
    }


    DPRINT(Debug,7,(&Debug, "initial_TLS_ra: proto=%s username=%s host=%s\n",
		    proto_name,
		    ra->username ? ra->username : "<NULL>",
		    ra->host ? ra->host : "<NULL>"));

    /* We must load all connect libraries */
    load_shared_libs1(&use_shared_connect); 

    for (i = 0; i < use_shared_connect.shared_list_len; i++) {
	if (use_shared_connect.shared_list[i]->handle) {
	    int r = give_rnum(use_shared_connect.shared_list[i],
			      &use_shared_connect);

	    struct sl_connect_data * connect =
		use_shared_connect.shared_list[i]->regs[r].r.connect;

	    if (SHARED_CONNECT_magic != connect->magic)
		panic("SHARED PANIC",__FILE__,__LINE__,
		      "initial_TLS_ra",
		      "Bad magic bumber",0);

	    if (use_shared_connect.shared_list[i]->regs[r].valid) {
		struct SE_option       *config = NULL;
		struct SE_option_type  *T      = NULL;
		int j;

		for (j = 0; j < shared_SE_option_type_count; j++) {
		    if (shared_SE_option_types[j].imp_idx ==
			use_shared_connect.shared_list[i]) {

			T = shared_SE_option_types[j].T;

			DPRINT(Debug,8,(&Debug,
					"connect library [%d] %s have service option type %p\n",
					i,use_shared_connect.shared_list[i]->
					shname,
					T));

			if (SE_option_t_magic != T->magic) 
			    panic("CONNECTION PANIC",__FILE__,__LINE__,
				  "initial_TLS_ra",
				  "Bad magic (SE_option_type)",0);
		    }
		}	

		if (ra->service) {
		    if (SERVICE_ENTRY_magic != ra->service->magic)
			panic("CONNECTION PANIC",__FILE__,__LINE__,
			      "initial_TLS_ra",
			      "Bad magic (service_entry)",0);

		    for (j = 0; j < ra->service->option_count; j++) {
			if (ra->service->option_list[j].type == T) {
			    config = & (ra->service->option_list[j]);
			    
			    DPRINT(Debug,8,(&Debug,
					    "  ... have service config %p (%s)\n",
					    config,
					    config->prefix ? config->prefix : "NONE"));
			}
		    }		
		}

		switch ((ret = 
			 connect->initial_TLS_ra(ra,proto_name,
						 config,
						 sizeof(* config)))) {

		case tls_ra_unsupported:
		    /* try next library */
		    break;
		    
		case tls_ra_failed:
		    DPRINT(Debug,8,(&Debug,
				    "connect library [%d] %s failed to initialize TLS for %s\n",
				    i,use_shared_connect.shared_list[i]->shname,
				    proto_name));
		    goto out;
		case tls_ra_none:
		    DPRINT(Debug,8,(&Debug,
				    "connect library [%d] %s didn't initialized TLS for %s\n",
				    i,use_shared_connect.shared_list[i]->shname,
				    proto_name));
		    sticky_ret = ret;
		    /* try next library */
		    break;
		case tls_ra_done:
		    DPRINT(Debug,8,(&Debug,
				    "connect library [%d] %s initialized TLS for %s\n",
				    i,use_shared_connect.shared_list[i]->shname,
				    proto_name));
		    goto out;
		default:
		    DPRINT(Debug,1,(&Debug,
				    "connect library [%d] %s returned unexpec status %d when initializing TLS for %s\n",
				    i,use_shared_connect.shared_list[i]->shname,
				    ret,proto_name));
		    goto out;
		}		
	    }
	}

    }

    ret = sticky_ret;

 out:

    /* must "translate" return */

    switch(ret) {
    case tls_ra_unsupported:
	DPRINT(Debug,7,(&Debug, "initial_TLS_ra=itls_unsupported, proto=%s\n",
			proto_name));
	return itls_unsupported;

    default:
	DPRINT(Debug,7,(&Debug, "initial_TLS_ra: bad return value %d\n",
			ret));
	/* FALLTHROUGH */
    case tls_ra_failed:
	DPRINT(Debug,7,(&Debug, "initial_TLS_ra=itls_failed, proto=%s\n",
			proto_name));
	return itls_failed;
    case tls_ra_none:
	DPRINT(Debug,7,(&Debug, "initial_TLS_ra=itls_none, proto=%s\n",
			proto_name));
	return itls_none;
    case tls_ra_done:
	DPRINT(Debug,7,(&Debug, "initial_TLS_ra=itls_done, proto=%s\n",
			proto_name));
	return itls_done;
    }
}
#endif

/*
 * Local Variables:
 *  mode:c
 *  c-basic-offset:4
 * End:
 */
